from contextlib import suppress
from pathlib import Path


def patch_promplate():
    import promplate

    class Loader(promplate.template.Loader):
        """making HTTP requests in pyodide runtime"""

        @classmethod
        async def afetch(cls, url: str, **kwargs):
            from pyodide.http import pyfetch

            res = await pyfetch(cls._join_url(url))
            obj = cls(await res.text())
            obj.name = Path(url).stem

            return obj

        @classmethod
        def fetch(cls, url: str, **kwargs):
            from pyodide.http import open_url

            res = open_url(cls._join_url(url))
            obj = cls(res.read())
            obj.name = Path(url).stem

            return obj

    class Node(Loader, promplate.Node):
        """patched for making HTTP requests in pyodide runtime"""

    class Template(Loader, promplate.Template):
        """patched for making HTTP requests in pyodide runtime"""

    promplate.template.Loader = Loader
    promplate.template.Template = promplate.Template = Template
    promplate.node.Node = promplate.Node = Node

    from promplate.prompt.chat import ensure as _ensure

    from .utils.proxy import to_js

    def ensure(text_or_list: list[promplate.Message] | str):
        """This function is patched to return a JS array. So it should not be called from Python."""
        return to_js(_ensure(text_or_list))

    promplate.prompt.chat.ensure = ensure

    with suppress(ModuleNotFoundError):
        import promplate.llm.openai.v1 as o

        del o.TextComplete, o.TextGenerate, o.ChatComplete, o.ChatGenerate, o.SyncTextOpenAI, o.SyncChatOpenAI

    promplate.prompt.chat.ensure = _ensure


async def patch_openai():
    from .utils.openai import ensure_openai, translate_openai

    await ensure_openai()

    import openai

    openai.Client = openai.AsyncClient = translate_openai()


def patch_httpx():
    from pyodide.code import run_js
    from pyodide.ffi import register_js_module

    register_js_module("httpx", run_js("({ Client() { return null }, AsyncClient() { return null } })"))


async def patch_all():
    await patch_openai()
    patch_httpx()
    patch_promplate()
